import pytest

from django_insights.registry import registry


@pytest.mark.django_db(True)
def collect_insights() -> None:
    from django_insights.metrics import metrics

    registry.autodiscover_insights()
    metrics.collect()



# Generated by Django 4.1.7 on 2023-03-28 07:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('created', models.DateTimeField(verbose_name='created')),
                (
                    'modified',
                    models.DateTimeField(auto_now=True, verbose_name='modified'),
                ),
                ('gender', models.IntegerField(choices=[('M', 1), ('F', 2)])),
                ('name', models.CharField(max_length=32, unique=True)),
                ('age', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Book',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('created', models.DateTimeField(verbose_name='created')),
                (
                    'modified',
                    models.DateTimeField(auto_now=True, verbose_name='modified'),
                ),
                ('title', models.CharField(max_length=128)),
                (
                    'author',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='books',
                        to='testapp.author',
                    ),
                ),
            ],
        ),
    ]



from __future__ import annotations

from typing import Any

from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.views.generic import DetailView, ListView, View

from django_insights.charts import barchart, scatterplot, timeseries, to_bytes_io
from django_insights.models import App, Bucket, Counter, Gauge
from django_insights.settings import settings


class InsightAppMixin(View):
    apps: list[App] = []

    def get_apps(self) -> QuerySet[App]:
        if not self.apps:
            self.apps = App.objects.all()
        return self.apps

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context.update(
            {'app_name': settings.INSIGHTS_APP_NAME, 'apps': self.get_apps()}
        )
        return context


class InsightsAppView(InsightAppMixin, DetailView):
    model = App
    slug_field = 'uuid'
    slug_url_kwarg = 'app_uuid'
    template_name = "insights/app.html"


class InsightsDashboardView(InsightAppMixin, ListView):
    template_name = "insights/dashboard.html"

    def get_queryset(self) -> list[Any]:
        return Counter.objects.all()

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context.update({'gauges': Gauge.objects.all()})
        return context


class InsightsChartView(View):
    async def get(self, request, bucket_id):
        theme = self.request.COOKIES.get('theme') or settings.INSIGHTS_DEFAULT_THEME
        bucket = await Bucket.objects.aget(pk=bucket_id)

        fig = None

        if bucket.is_timeseries:
            fig = await timeseries(bucket, theme=theme)
        if bucket.is_scatterplot:
            fig = await scatterplot(bucket, theme=theme)
        if bucket.is_barchart:
            fig = await barchart(bucket, theme=theme)

        buffer = to_bytes_io(fig)

        return HttpResponse(buffer, content_type='image/png')



from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models
from django.utils import timezone


class AppUser(AbstractBaseUser, PermissionsMixin):
    created = models.DateTimeField('created', auto_now_add=True)
    modified = models.DateTimeField('modified', auto_now=True)

    email = models.EmailField(blank=True, default="", unique=True)
    last_login = models.DateTimeField(blank=True, null=True)
    date_joined = models.DateTimeField(default=timezone.now)

    USERNAME_FIELD = "email"
    EMAIL_FIELD = "email"
    REQUIRED_FIELDS = []



import pytest
from django.core import management
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, override_settings

from django_insights.database import check_settings
from django_insights.models import Counter
from django_insights.registry import registry
from project.settings.test import DATABASES
from project.testapp.users.models import AppUser
from tests.utils import collect_insights

COPY_DATABASES = DATABASES.copy()
COPY_DATABASES.pop('insights')


class ApiTests(TestCase):
    databases = ['default', 'insights']

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        management.call_command('seed_db')
        collect_insights()

    @override_settings(DATABASES=COPY_DATABASES)
    def test_throw_error_no_insights_db_configured(self):
        with pytest.raises(ImproperlyConfigured):
            check_settings()

    def test_autodiscover_registry(self):
        assert registry.registered_insights

    def test_one_app_user_found(self):
        assert AppUser.objects.count() == 1

    def test_count_authors(self):
        counter = Counter.objects.get(label="count_authors")
        assert counter.value == 2000

    def test_count_books(self):
        counter = Counter.objects.get(label="count_books")
        assert counter.value == 5000



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

app_labels = {"insights"}
database_entry = "insights"


def check_settings():
    if database_entry not in settings.DATABASES:
        raise ImproperlyConfigured()


check_settings()


class Router:
    def db_for_read(self, model, **hints):
        if model._meta.app_label in app_labels:
            return "insights"
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label in app_labels:
            return "insights"

    def allow_relation(self, obj1, obj2, **hints):
        if obj1._meta.app_label in app_labels or obj2._meta.app_label in app_labels:
            return True
        return None

    def allow_migrate(self, db, app_label, **hints):
        if app_label in app_labels:
            return db == database_entry
        return None



from __future__ import annotations

from typing import Any, Callable

from django.utils.module_loading import autodiscover_modules
from tqdm import tqdm


class InsightRegistry:
    registered_insights: list[tuple[str, str, str, Callable[[Any], Any]]] = list()

    @staticmethod
    def autodiscover_insights():
        autodiscover_modules("insights")

    def register_insight(
        self,
        label: str,
        module: str,
        question: str,
        func: Callable[[Any], Any],
    ):
        self.registered_insights.append((module, label, question, func))

    def collect_insights(self):
        progress_iterator = tqdm(self.registered_insights, desc='Collect insights')

        for module, name, question, metric in progress_iterator:
            progress_iterator.set_description(
                desc=f"Create insights for {module}.{name}", refresh=True
            )
            metric()

        progress_iterator.set_description(desc="Done!")


registry = InsightRegistry()



class BucketType:
    TIMESERIES = 1
    HISTOGRAM = 2
    SCATTERPLOT = 3
    BARCHART = 4
    BUCKET_TYPES = (
        (TIMESERIES, 'timeseries'),
        (HISTOGRAM, 'histogram'),
        (SCATTERPLOT, 'scatterplot'),
        (BARCHART, 'barchart'),
    )



# Generated by Django 4.1.7 on 2023-03-28 07:45

import uuid

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='App',
            fields=[
                (
                    'uuid',
                    models.UUIDField(
                        default=uuid.uuid4,
                        primary_key=True,
                        serialize=False,
                        unique=True,
                    ),
                ),
                (
                    'module',
                    models.CharField(db_index=True, max_length=254, unique=True),
                ),
                ('label', models.CharField(max_length=254, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Bucket',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created_at',
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now
                    ),
                ),
                ('label', models.CharField(db_index=True, max_length=254, unique=True)),
                ('question', models.TextField(blank=True, null=True)),
                ('desc', models.TextField(blank=True, null=True)),
                (
                    'type',
                    models.IntegerField(
                        choices=[
                            (1, 'timeseries'),
                            (2, 'histogram'),
                            (3, 'scatterplot'),
                            (4, 'barchart'),
                        ],
                        default=1,
                    ),
                ),
                ('xlabel', models.CharField(blank=True, max_length=254, null=True)),
                ('xformat', models.CharField(blank=True, max_length=254, null=True)),
                ('ylabel', models.CharField(blank=True, max_length=254, null=True)),
                ('yformat', models.CharField(blank=True, max_length=254, null=True)),
                ('title', models.CharField(blank=True, max_length=254, null=True)),
                (
                    'app',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='buckets',
                        to='insights.app',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='ExecutionDelta',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'executed_at',
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now
                    ),
                ),
                ('uuid', models.UUIDField(default=uuid.uuid4)),
            ],
        ),
        migrations.CreateModel(
            name='Gauge',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created_at',
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now
                    ),
                ),
                ('label', models.CharField(db_index=True, max_length=254, unique=True)),
                ('question', models.TextField(blank=True, null=True)),
                ('desc', models.TextField(blank=True, null=True)),
                ('value', models.FloatField(default=0.0)),
                (
                    'app',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='gauges',
                        to='insights.app',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='Counter',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created_at',
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now
                    ),
                ),
                ('label', models.CharField(db_index=True, max_length=254, unique=True)),
                ('question', models.TextField(blank=True, null=True)),
                ('desc', models.TextField(blank=True, null=True)),
                ('value', models.IntegerField(default=0)),
                (
                    'app',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='counters',
                        to='insights.app',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='BucketValue',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'timestamp',
                    models.DateTimeField(blank=True, db_index=True, null=True),
                ),
                ('xvalue', models.FloatField(default=0.0)),
                ('yvalue', models.FloatField(default=0.0)),
                (
                    'category',
                    models.CharField(db_index=True, max_length=254, null=True),
                ),
                (
                    'bucket',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='values',
                        to='insights.bucket',
                    ),
                ),
            ],
        ),
        migrations.AddConstraint(
            model_name='gauge',
            constraint=models.UniqueConstraint(
                fields=('app', 'label'), name='gauge_unique_app_label'
            ),
        ),
        migrations.AddConstraint(
            model_name='counter',
            constraint=models.UniqueConstraint(
                fields=('app', 'label'), name='counter_unique_app_label'
            ),
        ),
        migrations.AddConstraint(
            model_name='bucketvalue',
            constraint=models.UniqueConstraint(
                fields=('bucket', 'timestamp'),
                name='bucketvalue_unique_bucket_timestamp',
            ),
        ),
        migrations.AddConstraint(
            model_name='bucket',
            constraint=models.UniqueConstraint(
                fields=('app', 'label'), name='bucket_unique_app_label_timestamp'
            ),
        ),
    ]
